const HtmlWebpackPlugin = require('html-webpack-plugin');
const MiniCssExtractPlugin = require("mini-css-extract-plugin");
const {
  getFileNames,
  ownDir,
  getPublicPath
} = require('./utils');
const path = require('path');
const cssLoader = require('./webpack/css-loader');
const imgLoader = require('./webpack/img-loader');
const eslintLoader = require('./webpack/eslint-loader');

module.exports = ({
  cwd = process.cwd(),
  mode,
  entry = './src/',
  dist = './dist/',
  homepage,
  filename,
  format,
  moduleName,
  hash,
  html = {},
  cssModule = true,
  extractCss = true,
  extractCssName = 'style.css',
  inlineImageMaxSize = 8192
} = {}) => {
  const config = {};
  // mode
  config.mode = mode;

  // entry
  config.entry = {
    app: mode === 'development' ? [entry, 'webpack-dev-server/client?http://localhost:4255/'] : [entry]
  };

  // output
  const useHash = typeof hash === 'boolean' ? hash : (mode === 'production' && !format);
  const filenames = getFileNames(useHash, filename);

  config.output = {
    path: path.resolve(cwd, dist),
    filename: filenames.js,
    publicPath: getPublicPath(mode, homepage)
  };
  
  if (format === 'cjs') {
    config.output.libraryTarget = 'commonjs2';
  } else if (format === 'amd'){
    config.output.libraryTarget = 'amd';
  } else if (format === 'umd') {
    config.output.library = moduleName;
    config.output.libraryTarget = 'umd';
  } else if (format === 'iife') {
    config.output.libraryTarget = 'window';
  }

  // resolve
  config.resolve = {
    alias: {
      '@': path.resolve(cwd, 'src')
    },
    extensions: ['.js', '.json'],
    modules: [path.resolve(cwd, 'node_modules'), 'node_modules', ownDir('node_modules')]
  }

  // loader
  config.module = {
    rules: []
  };
  // loader -> eslint
  config.module.rules.push(eslintLoader());
  // loader -> css
  config.module.rules = config.module.rules.concat(cssLoader(mode, cssModule, extractCss));
  // lodaer -> img
  config.module.rules.push(imgLoader(inlineImageMaxSize));

  // optimization
  config.optimization = {};
  // optimization -> splitChunks
  if (mode === 'production' && !format) {
    config.optimization.splitChunks = {
      cacheGroups: {
        commons: {
          test: /[\\/]node_modules[\\/]/,
          name: "vendors",
          chunks: "all"
        }
      }
    };
  }

  // plugin
  config.plugins = [];
  // plugin -> html
  if ((html || mode === 'development') && !format) {
    html = html || {};
    const defaultHtml = {
      title: 'Mints',
      template: ownDir('lib/template.ejs')
    }
    config.plugins.push(new HtmlWebpackPlugin(Object.assign({}, defaultHtml, html)));
  }
  // plugin -> extract
  if (extractCss && mode === 'production') {
    config.plugins.push(new MiniCssExtractPlugin({
      filename: extractCssName
    }));
  }

  return config;
}